<?php

namespace Concrete\Package\ToessLabRobotsTxt\Controller\SinglePage\Dashboard\System\Seo;
use Concrete\Core\Http\ResponseFactoryInterface;
use \Concrete\Core\Page\Controller\DashboardPageController;
use Concrete\Core\Url\Resolver\Manager\ResolverManagerInterface;
use Concrete\Package\ToessLabRobotsTxt\RobotTxt\RobotTxt;
use Illuminate\Filesystem\Filesystem;

defined('C5_EXECUTE') or die("Access Denied.");

class EditRobotsTxt extends DashboardPageController
{

    /**
     * @var string
     */
    protected $robotsTxtContent;

    /**
     * @var RobotTxt
     */
    protected $robotsTxt;

    /**
     * @var Filesystem
     */
    protected $fileSystem;

    /**
     * @var $session
     */
    protected $session;

    /**
     * @var ResponseFactoryInterface
     */
    protected $responseFactory;

    /**
     * @var ResolverManagerInterface
     */
    protected $resolverManager;

    public function view()
    {
        $this->set('versions', $this->robotsTxt->getVersions());
        $this->set('robotsTxt', $this->checkContent());
        $this->requireAsset('toess_lab_robots_txt');
    }

    public function on_start()
    {
        parent::on_start();
        $this->robotsTxt = new RobotTxt();
        $this->fileSystem = $this->robotsTxt->getFileSystem();
        $this->robotsTxt->checkOriginal();
        $this->session = $this->app->make('session');
        $this->responseFactory = $this->app->make(ResponseFactoryInterface::class);
        $this->resolverManager = $this->app->make(ResolverManagerInterface::class);
        $this->set('formHelper', $this->app->make('helper/form'));
        $this->set('dateHelper', $this->app->make('helper/date'));
        $this->requireAsset('toess_lab_robots_txt');
    }

    /**
     * @return \Symfony\Component\HttpFoundation\Response
     * @throws \Exception
     */
    public function save_robot()
    {
        $securityService = $this->app->make('helper/security');
        $newText = $securityService->sanitizeString($this->request->post('robots_txt'));
        $this->save($newText);
        return $this->responseFactory->redirect(
            $this->resolverManager->resolve(['/dashboard/system/seo/edit_robots_txt/']),
            302
        );
    }

    /**
     * @param bool $return
     * @return \Symfony\Component\HttpFoundation\Response
     * @throws \Illuminate\Contracts\Filesystem\FileNotFoundException
     */
    public function restore_original($return = false)
    {
        $this->robotsTxtContent = $this->robotsTxt->getFromConfig(-1);
        $this->saveFileOnly($this->robotsTxtContent);
        if ($return) {
            $this->flash('success', t('All robots.txt versions have been deleted. The original robots.txt has been restored'));
        } else {
            $this->flash('success', t('The original robots.txt has been restored'));
        }
        return $this->responseFactory->redirect(
            $this->resolverManager->resolve(['/dashboard/system/seo/edit_robots_txt/']),
            302
        );
    }

    /**
     * @param int $v
     * @return \Symfony\Component\HttpFoundation\Response
     * @throws \Illuminate\Contracts\Filesystem\FileNotFoundException
     */
    public function restore_version($v = 0)
    {
        $this->robotsTxtContent = $this->robotsTxt->getFromConfig((int)$v);
        $this->saveFileOnly($this->robotsTxtContent);
        $this->flash('success', t('The robots.txt has been restored to version: %s', $this->robotsTxt->getOneVersion($v)));
        return $this->responseFactory->redirect(
            $this->resolverManager->resolve(['/dashboard/system/seo/edit_robots_txt/']),
            302
        );
    }

    /**
     * @param bool $v
     * @return \Symfony\Component\HttpFoundation\Response
     * @throws \Illuminate\Contracts\Filesystem\FileNotFoundException
     */
    public function delete_version($v = false)
    {
        $this->robotsTxt->deleteFromConfig((int)$v);
        $this->flash('success', t('The robots.txt version %s has been deleted', $this->formatDate($v)) . '\n' . t('The original robots.txt has been restored'));
        $this->restore_original();
        return $this->responseFactory->redirect(
            $this->resolverManager->resolve(['/dashboard/system/seo/edit_robots_txt/']),
            302
        );
    }

    /**
     * @return \Symfony\Component\HttpFoundation\Response
     * @throws \Illuminate\Contracts\Filesystem\FileNotFoundException
     */
    public function delete_all()
    {
        $this->robotsTxt->deleteAllFromConfig();
        return $this->restore_original(true);
    }

    /**
     * @param $timestamp
     * @return mixed
     * @throws \Exception
     */
    public function formatDate($timestamp)
    {
        $versionDate = new \DateTime();
        $versionDate->setTimestamp($timestamp);
        return $this->app->make('helper/date')->formatDateTime($versionDate);
    }

    /**
     * @param $content
     * @return bool
     * @throws \Exception
     */
    protected function save($content)
    {
        $timestamp = new \DateTime();
        $version = $timestamp->getTimestamp();
        if ($this->robotsTxt->getFileSystem()->isWritable($this->robotsTxt->getPath())) {
            $this->robotsTxt->getFileSystem()->put('robots.txt', $content);
            $this->robotsTxtContent = $this->robotsTxt->saveVersionToConfig($content, $version);
        } else {
            $this->flash('error', t('The robots.txt could not be saved! Please check permissions'));
            return false;
        }
        $this->flash('success', t('The robots.txt has been saved as version %s', $this->formatDate($version)));
    }

    /**
     * @param $content
     * @return bool
     */
    protected function saveFileOnly($content)
    {
        if ($this->robotsTxt->getFileSystem()->isWritable($this->robotsTxt->getPath())) {
            $this->robotsTxt->getFileSystem()->put('robots.txt', str_replace("\\n","\n", $content));
        } else {
            $this->flash('error', t('The robots.txt could not be saved! Please check permissions'));
            return false;
        }
    }

    /**
     * @return string
     * @throws \Illuminate\Contracts\Filesystem\FileNotFoundException
     */
    protected function checkContent()
    {
        return (is_null($this->robotsTxtContent)) ? $this->robotsTxt->getFromConfig() : $this->robotsTxtContent;
    }
}