<?php
namespace Concrete\Package\ToessLabRobotsTxt\RobotTxt;

use Concrete\Core\Support\Facade\Application;
use Concrete\Core\Support\Facade\Database;
use Illuminate\Filesystem\Filesystem;

defined('C5_EXECUTE') or die("Access Denied.");

class RobotTxt
{
    /**
     * @var string
     */
    protected $robotsTxtPath = 'robots.txt';

    /**
     * @var Filesystem
     */
    protected $fileSystem;

    /**
     * @var $configDB
     */
    protected $configDB;

    /**
     * @var $helperDate
     */
    protected $helperDate;

    /**
     * @var string
     */
    protected $originalRobotsTxtContent = 'User-agent: *\nDisallow: /application/attributes\nDisallow: /application/authentication\nDisallow: /application/bootstrap\nDisallow: /application/config\nDisallow: /application/controllers\nDisallow: /application/elements\nDisallow: /application/helpers\nDisallow: /application/jobs\nDisallow: /application/languages\nDisallow: /application/mail\nDisallow: /application/models\nDisallow: /application/page_types\nDisallow: /application/single_pages\nDisallow: /application/tools\nDisallow: /application/views\\nDisallow: /ccm/system/captcha/picture';

    /**
     * @var array $versions
     */
    protected $versions = [];

    public function __construct()
    {
        $this->fileSystem = new Filesystem();
        $app = Application::getFacadeApplication();
        $this->configDB = $app->make('config/database');
        $this->helperDate = $app->make('helper/date');
    }

    /**
     * @return Filesystem
     */
    public function getFileSystem()
    {
        return $this->fileSystem;
    }

    /**
     *
     */
    public function checkOriginal()
    {
        if (!$this->fileSystem->exists($this->robotsTxtPath)) {
            $this->createStandard();
        }
    }

    /**
     *
     */
    public function checkOriginalExists()
    {
        return ($this->fileSystem->exists($this->robotsTxtPath));
    }

    /**
     * @param int $v
     * @return string
     * @throws \Illuminate\Contracts\Filesystem\FileNotFoundException
     */
    public function getFromConfig($v = 0)
    {
        $version = 'version.' . $v;
        if ($v == 0) {
            return $this->fileSystem->get($this->robotsTxtPath);
        }
        if ($v == -1) {
            $version = 'original';
        }
        return $this->configDB->get('toess_lab_robots_txt.' . $version);

    }

    /**
     * @param int $v
     * @return mixed
     */
    public function deleteFromConfig($v = 0)
    {
        $db = Database::connection();
        $ps = $db->prepare('DELETE FROM `Config` WHERE  `configNamespace`="" AND `configGroup`="toess_lab_robots_txt" AND `configItem`=:version;');
        $ps->execute([':version' => 'version.' . $v]);
    }

    /**
     * @return mixed
     */
    public function deleteAllFromConfig()
    {
        $db = Database::connection();
        $ps = $db->prepare('DELETE FROM `Config` WHERE  `configNamespace`="" AND `configGroup`="toess_lab_robots_txt" AND `configItem` like "version.%";');
        $ps->execute();
    }

    /**
     * @return string
     */
    public function getPath()
    {
        return $this->robotsTxtPath;
    }

    /**
     * @param string $content
     * @return string
     */
    public function saveOriginalToConfig($content = '')
    {
        if ($content === '' || is_null($content)) {
            $content = $this->originalRobotsTxtContent;
        }
        $this->configDB->save('toess_lab_robots_txt.original', $content);
        return $content;
    }

    /**
     * @param string $content
     * @param        $version
     * @return string
     */
    public function saveVersionToConfig($content = '', $version = false)
    {
        $version = 'version.' . $version;
        if ($content === '' || is_null($content)) {
            $content = $this->originalRobotsTxtContent;
        }
        if (!$version) {
            $version = 'original';
        }
        $this->configDB->save('toess_lab_robots_txt.' . $version, $content);
        return $content;
    }

    /**
     * @return array
     * @throws \Exception
     */
    public function getVersions()
    {
        $this->versions = $this->configDB->get('toess_lab_robots_txt.version');
        $formattedDateArray = [];
        $formattedDateReturnArray = [];
        if (is_array($this->versions)) {
            foreach($this->versions as $key => $v) {
                $versionDate = new \DateTime();
                $versionDate->setTimestamp($key);
                $formattedDateArray[$key] = $this->helperDate->formatDateTime($versionDate);
                $duplicates = array_count_values($formattedDateArray);
                $d = 0;
                foreach ($duplicates as $k => $duple) {
                    for($i = 0; $i < $duple; $i++) {
                        $val = ($i > 0) ? ' (' . $i . ')' : '';
                        $formattedDateReturnArray[$key] = $k . $val;
                        $d++;
                    }
                }
            }
        }
        krsort($formattedDateReturnArray);
        return $formattedDateReturnArray;
    }

    /**
     * @param $v
     * @return mixed
     * @throws \Exception
     */
    public function getOneVersion($v)
    {
        $versions = $this->getVersions();
        return $versions[$v];
    }

    /**
     *
     */
    protected function createStandard()
    {
        $this->configDB->save('toess_lab_robots_txt.original', str_replace("\\n","\n", $this->originalRobotsTxtContent));
        $this->fileSystem->put($this->robotsTxtPath, str_replace("\\n","\n", $this->originalRobotsTxtContent));
    }

}